#include <avr/interrupt.h>
#include <avr/sleep.h>

#define VOLTAGE_MONITOR_PIN           A0
#define CHARGE_PUMP_PULSE_TRAIN_PIN1  8
#define CHARGE_PUMP_PULSE_TRAIN_PIN2  9
#define ONBOARD_LED                   13

#define NUM_READINGS    5

static uint16_t reading[NUM_READINGS]; 

void setup()
{
  analogReference(INTERNAL);
  
  pinMode(VOLTAGE_MONITOR_PIN, INPUT);
  pinMode(CHARGE_PUMP_PULSE_TRAIN_PIN1, OUTPUT);
  pinMode(CHARGE_PUMP_PULSE_TRAIN_PIN2, OUTPUT);
  pinMode(ONBOARD_LED, INPUT);

  digitalWrite(CHARGE_PUMP_PULSE_TRAIN_PIN1, LOW);
  digitalWrite(CHARGE_PUMP_PULSE_TRAIN_PIN2, HIGH);

  //We want to start with max ADC values so that we don't turn off
  //After 5 readings, we will have a better idea of what the battery voltage is
  for(int r=0; r<NUM_READINGS; r++)
  {
    reading[r] = 0x3ff;
  }

  cli();
  
  // Setup Timer 2 to create an overflow interrupt every ~5ms
  TCCR2A = 0;
  TCCR2B = 0x05; // 1/128 prescalar divider
  TIMSK2 = 0x01; // Enable overflow interrupt
  TIFR2 = 0x01; // Clear the overflow flag
  
  set_sleep_mode(SLEEP_MODE_PWR_SAVE);
  
  sei();
}

void loop()
{
  static uint8_t r = 0;

  // Take a new reading
  reading[r] = analogRead(VOLTAGE_MONITOR_PIN);

  // Increment r for the next time we wake up
  r++;
  if(r >= NUM_READINGS)
  {
    r=0;
  }

  uint16_t total = 0;
  for(int t=0; t<NUM_READINGS; t++)
  {
    total += reading[t];
  }
  
  if (total < NUM_READINGS * 10240/11 * 3*330/1330) //This is the reading when V_battery = 3V (using nominal values)
  {
    // Turn everything off
    PORTB = 0;
    
    // Go to sleep forever (and turn off the BOD on our way)
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);
    cli(); // Turn off all interrupts so we never wake up
    sleep_enable();
    sleep_bod_disable();
    sleep_cpu();

    // THE END
  }
  else
  {
    // Keep the charge pump on
    PORTB ^= 0x03;

    // Go to sleep until the next timer overflow
    sleep_mode();
  }
}

EMPTY_INTERRUPT(TIMER2_OVF_vect); //Do nothing. We just need to wake up the CPU every ~5ms

